@@ -16,6 +16,7 @@
 
 #include "alligatorsettings.h"
 #include "database.h"
+#include "debug.h"
 #include "fetcher.h"
 
 #define TRUE_OR_RETURN(x)                                                                                                                                      \
@@ -29,11 +30,11 @@ Database::Database()
     QDir(databasePath).mkpath(databasePath);
     db.setDatabaseName(databasePath + QStringLiteral("/database.db3"));
     if (!db.open()) {
-        qCritical() << "Failed to open the database";
+        qCCritical(ALLIGATOR) << "Failed to open the database";
     }
 
-    if (!migrateTo(2)) {
-        qCritical() << "Failed to migrate the database";
+    if (!migrateTo(3)) {
+        qCritical(ALLIGATOR) << "Failed to migrate the database";
     }
 
     cleanup();
@@ -42,7 +43,7 @@ Database::Database()
 bool Database::migrateTo(const int targetVersion)
 {
     if (version() >= targetVersion) {
-        qDebug() << "Database already in version" << targetVersion;
+        qCDebug(ALLIGATOR) << "Database already in version" << targetVersion;
         return true;
     }
 
@@ -51,16 +52,28 @@ bool Database::migrateTo(const int targetVersion)
         return migrateTo1();
     case 2:
         return migrateTo2();
+    case 3:
+        return migrateTo3();
     default:
         return true;
     }
 }
 
+bool Database::migrateTo3()
+{
+    migrateTo2();
+
+    qDebug() << "Migrating database to version 3";
+    TRUE_OR_RETURN(execute(QStringLiteral("ALTER TABLE Entries ADD COLUMN favorite BOOL DEFAULT FALSE;")));
+    TRUE_OR_RETURN(execute(QStringLiteral("PRAGMA user_version = 3;")));
+    return true;
+}
+
 bool Database::migrateTo2()
 {
     migrateTo(1);
 
-    qDebug() << "Migrating database to version 2";
+    qCDebug(ALLIGATOR) << "Migrating database to version 2";
     TRUE_OR_RETURN(execute(QStringLiteral("CREATE TABLE IF NOT EXISTS FeedGroups (name TEXT NOT NULL, description TEXT, defaultGroup INTEGER);")));
     TRUE_OR_RETURN(execute(QStringLiteral("ALTER TABLE Feeds ADD COLUMN groupName TEXT;")));
     TRUE_OR_RETURN(execute(QStringLiteral("ALTER TABLE Feeds ADD COLUMN displayName TEXT;")));
@@ -74,7 +87,7 @@ bool Database::migrateTo2()
 
 bool Database::migrateTo1()
 {
-    qDebug() << "Migrating database to version 1";
+    qCDebug(ALLIGATOR) << "Migrating database to version 1";
     TRUE_OR_RETURN(
         execute(QStringLiteral("CREATE TABLE IF NOT EXISTS Feeds (name TEXT, url TEXT, image TEXT, link TEXT, description TEXT, deleteAfterCount INTEGER, "
                                "deleteAfterType INTEGER, subscribed INTEGER, lastUpdated INTEGER, notify BOOL);")));
@@ -97,9 +110,9 @@ bool Database::execute(const QString &query)
 bool Database::execute(QSqlQuery &query)
 {
     if (!query.exec()) {
-        qWarning() << "Failed to execute SQL Query";
-        qWarning() << query.lastQuery();
-        qWarning() << query.lastError();
+        qCWarning(ALLIGATOR) << "Failed to execute SQL Query";
+        qCWarning(ALLIGATOR) << query.lastQuery();
+        qCWarning(ALLIGATOR) << query.lastError();
         return false;
     }
     return true;
@@ -114,7 +127,7 @@ int Database::version()
         return -1;
     }
     int value = query.value(0).toInt();
-    qDebug() << "Database version " << value;
+    qCDebug(ALLIGATOR) << "Database version " << value;
     return value;
 }
 
@@ -141,7 +154,7 @@ void Database::cleanup()
         qint64 sinceEpoch = dateTime.toSecsSinceEpoch();
 
         QSqlQuery query;
-        query.prepare(QStringLiteral("DELETE FROM Entries WHERE updated < :sinceEpoch;"));
+        query.prepare(QStringLiteral("DELETE FROM Entries WHERE updated < :sinceEpoch AND favorite != 1;"));
         query.bindValue(QStringLiteral(":sinceEpoch"), sinceEpoch);
         execute(query);
     }
@@ -159,12 +172,12 @@ bool Database::feedExists(const QString &url)
 
 void Database::addFeed(const QString &url, const QString &groupName, const bool markEntriesRead)
 {
-    qDebug() << "Adding feed";
+    qCDebug(ALLIGATOR) << "Adding feed";
     if (feedExists(url)) {
-        qDebug() << "Feed already exists";
+        qCDebug(ALLIGATOR) << "Feed already exists";
         return;
     }
-    qDebug() << "Feed does not yet exist";
+    qCDebug(ALLIGATOR) << "Feed does not yet exist";
 
     QUrl urlFromInput = QUrl::fromUserInput(url);
     QSqlQuery query;
@@ -233,7 +246,7 @@ void Database::exportFeeds(const QString &path)
 void Database::addFeedGroup(const QString &name, const QString &description, const int isDefault)
 {
     if (feedGroupExists(name)) {
-        qDebug() << "Feed group already exists, nothing to add";
+        qCDebug(ALLIGATOR) << "Feed group already exists, nothing to add";
         return;
     }
 
@@ -282,6 +295,38 @@ void Database::setRead(const QString &entryId, bool read)
     Q_EMIT entryReadChanged(entryId, read);
 }
 
+void Database::setFeedRead(const QString &url, bool read)
+{
+    QSqlQuery query;
+    query.prepare(QStringLiteral("UPDATE Entries SET read=:read WHERE feed=:url"));
+    query.bindValue(QStringLiteral(":url"), url);
+    query.bindValue(QStringLiteral(":read"), read);
+    execute(query);
+
+    Q_EMIT feedReadChanged();
+}
+
+void Database::setAllRead(bool read)
+{
+    QSqlQuery query;
+    query.prepare(QStringLiteral("UPDATE Entries SET read=:read"));
+    query.bindValue(QStringLiteral(":read"), read);
+    execute(query);
+
+    Q_EMIT feedReadChanged();
+}
+
+void Database::setFavorite(const QString &entryId, bool favorite)
+{
+    QSqlQuery query;
+    query.prepare(QStringLiteral("UPDATE Entries SET favorite=:favorite WHERE id=:id"));
+    query.bindValue(QStringLiteral(":id"), entryId);
+    query.bindValue(QStringLiteral(":favorite"), favorite);
+    execute(query);
+
+    Q_EMIT entryFavoriteChanged(entryId, favorite);
+}
+
 bool Database::feedGroupExists(const QString &name)
 {
     QSqlQuery query;
